//
// Copyright (c) 2015, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;


namespace MindFusion.Gauges.WinForms.Samples.CS.Clock
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			UpdateTime();
		}

		private void clock_PrepaintPointer(object sender, PrepaintEventArgs e)
		{
			e.CancelDefaultPainting = true;

			Brush fill = new SolidBrush(Color.FromArgb(94, 98, 99));
            GraphicsState state = e.Graphics.Save();
            e.Graphics.ScaleTransform(e.Element.RenderSize.Width, e.Element.RenderSize.Height);

            if (e.Element.Name == "HourHand")
			{
				e.Graphics.FillRectangle(fill, 0, 0.3f, 1f, 0.4f);
				e.Graphics.FillEllipse(fill, 0, 0, 0.08f, 1f);
			}
			else if (e.Element.Name == "MinuteHand")
			{
				e.Graphics.FillRectangle(fill, 0, 0.3f, 1f, 0.4f);
				e.Graphics.FillEllipse(fill, 0, 0, 0.05f, 1f);
			}
			else if (e.Element.Name == "SecondHand")
			{
				e.Graphics.FillRectangle(fill, 0, 0.4f, 1f, 0.2f);
				e.Graphics.FillEllipse(fill, 0, 0, 0.04f, 1f);
			}

            e.Graphics.Restore(state);
			fill.Dispose();
		}

		private void clock_PrepaintBackground(object sender, PrepaintEventArgs e)
		{
			e.CancelDefaultPainting = true;

			RectangleF bounds = new RectangleF(0, 0, e.Element.RenderSize.Width, e.Element.RenderSize.Height);

			// ---------------------- Ellipse #1 ----------------------
			Pen stroke1 = new Pen(Color.FromArgb(122, 123, 124), 0.01f * bounds.Width);
			Brush fill1 = new LinearGradientBrush(new PointF(0, 0.5f), new PointF(1, 0.5f),
				Color.FromArgb(245, 245, 245), Color.FromArgb(151, 156, 160));

			// Inflate with half the stroke thickness
			bounds.Inflate(-stroke1.Width / 2, -stroke1.Width / 2);

            e.Graphics.FillEllipse(fill1, bounds);
            e.Graphics.DrawEllipse(stroke1, bounds);

			fill1.Dispose();
			stroke1.Dispose();

			// Apply margin of 0.025
			bounds.Inflate(-0.025f * bounds.Width, -0.025f * bounds.Height);

			// ---------------------- Ellipse #2 ----------------------
			GraphicsPath fill2Path = new GraphicsPath();
			fill2Path.AddEllipse(bounds);

			PathGradientBrush fill2 = new PathGradientBrush(fill2Path);
			fill2.CenterPoint = new PointF(bounds.Bottom, bounds.Right);//new PointF(1, 1);

			ColorBlend fill2Blend = new ColorBlend();
			fill2Blend.Positions = new float[] { 0, 0.3f, 0.7f, 1 };
			fill2Blend.Colors = new Color[] { Color.FromArgb(225, 236, 238), Color.FromArgb(225, 236, 235), Color.White, Color.White };
			fill2.InterpolationColors = fill2Blend;

			e.Graphics.FillEllipse(fill2, bounds);

			fill2.Dispose();
			fill2Path.Dispose();

			// ---------------------- Ellipse #3 ----------------------
			GraphicsPath fill3Path = new GraphicsPath();
			fill3Path.AddEllipse(bounds);

			PathGradientBrush fill3 = new PathGradientBrush(fill3Path);

			ColorBlend fill3Blend = new ColorBlend();
			fill3Blend.Positions = new float[] { 0, 0.03f, 0.09f, 0.13f, 1 };
			fill3Blend.Colors = new Color[] { Color.FromArgb(0, Color.Black), Color.FromArgb(0, Color.Black), Color.FromArgb(32, Color.Black), Color.FromArgb(0, Color.Black), Color.FromArgb(0, Color.Black) };
			fill3.InterpolationColors = fill3Blend;

            e.Graphics.FillEllipse(fill3, bounds);

			fill3.Dispose();
			fill3Path.Dispose();

			// ---------------------- Ellipse #4 ----------------------
			Pen stroke4 = new Pen(Color.FromArgb(124, 137, 154), 0.01f);

			// Inflate with half the stroke thickness
			bounds.Inflate(-stroke4.Width / 2, -stroke4.Width / 2);

			e.Graphics.DrawEllipse(stroke4, bounds);

			stroke4.Dispose();
		}

		private void clock_PrepaintForeground(object sender, PrepaintEventArgs e)
		{
			e.CancelDefaultPainting = true;
            RectangleF bounds = new RectangleF(0, 0, e.Element.RenderSize.Width, e.Element.RenderSize.Height);

			GraphicsPath areaPath = new GraphicsPath();
			areaPath.AddEllipse(bounds);

			MindFusion.Drawing.PathGradientBrush area1Fill = new MindFusion.Drawing.PathGradientBrush(Color.FromArgb(80, Color.White), Color.Transparent);
			area1Fill.Path = areaPath;

			MindFusion.Drawing.PathGradientBrush area2Fill = new MindFusion.Drawing.PathGradientBrush(Color.FromArgb(80, Color.White), Color.Transparent);
			area2Fill.Path = areaPath;

			ArcArea area1 = new ArcArea();
            area1.RelativeCoordinates = false;
			area1.StartAngle = 160;
			area1.EndAngle = -20;
			area1.Margin = new Thickness(0.03f);
			area1.Fill = area1Fill;

			ArcArea area2 = new ArcArea();
            area2.RelativeCoordinates = false;
            area2.StartAngle = 140;
			area2.EndAngle = -40;
			area2.Margin = new Thickness(0.03f);
			area2.Fill = area2Fill;

			e.PaintVisualElement(area1, e.Element.RenderSize);
            e.PaintVisualElement(area2, e.Element.RenderSize);

			areaPath.Dispose();
		}

		private void timer_Tick(object sender, EventArgs  e)
		{
			UpdateTime();
		}

		private void UpdateTime()
		{
			Pointer hourHand = clock.GetElementByName("HourHand") as Pointer;
			Pointer minuteHand = clock.GetElementByName("MinuteHand") as Pointer;
			Pointer secondHand = clock.GetElementByName("SecondHand") as Pointer;

			TimeSpan now = DateTime.Now.TimeOfDay;
			hourHand.Value = (float)((now.TotalHours * 5) % 60);
			minuteHand.Value = (float)(now.TotalMinutes % 60);
			secondHand.Value = now.Seconds;
		}
	}
}